@@ -11,21 +11,23 @@ from contextvars import ContextVar
 from typing import Any, Callable, Dict, List, Optional, Tuple, Type, Union
 
 from .loaders_dumpers import get_loader_exceptions, load_value
-from .namespace import is_empty_namespace, Namespace, split_key, split_key_leaf, split_key_root
+from .namespace import Namespace, split_key, split_key_leaf, split_key_root
 from .optionals import FilesCompleterMethod, get_config_read_mode
 from .type_checking import ArgumentParser, _ArgumentGroup
 from .typing import path_type
 from .util import (
+    change_to_path_dir,
     default_config_option_help,
     DirectedGraph,
-    ParserError,
     import_object,
-    change_to_path_dir,
-    NoneType,
     indent_text,
+    is_subclass,
+    iter_to_set_str,
+    lenient_check_context,
+    NoneType,
+    parse_value_or_config,
+    ParserError,
     Path,
-    _parse_value_or_config,
-    _issubclass,
 )
 
 
@@ -212,7 +214,7 @@ class ActionConfigFile(Action, FilesCompleterMethod):
 
     @staticmethod
     def apply_config(parser, cfg, dest, value) -> None:
-        with _ActionSubCommands.not_single_subcommand():
+        with _ActionSubCommands.not_single_subcommand(), previous_config_context(cfg):
             if dest not in cfg:
                 cfg[dest] = []
             kwargs = {'env': False, 'defaults': False, '_skip_check': True, '_fail_no_subcommand': False}
@@ -232,6 +234,18 @@ class ActionConfigFile(Action, FilesCompleterMethod):
             cfg.update(cfg_file)
 
 
+previous_config: ContextVar = ContextVar('previous_config', default=None)
+
+
+@contextmanager
+def previous_config_context(cfg):
+    token = previous_config.set(cfg)
+    try:
+        yield
+    finally:
+        previous_config.reset(token)
+
+
 print_config_skip: ContextVar = ContextVar('print_config_skip', default=False)
 
 
@@ -245,10 +259,10 @@ class _ActionPrintConfig(Action):
                          default=default,
                          nargs=1,
                          metavar='[={comments,skip_null,skip_default}+]',
-                         help='Print configuration and exit.')
+                         help='Print the configuration after applying all other arguments and exit.')
 
     def __call__(self, parser, namespace, value, option_string=None):
-        kwargs = {'subparser': parser, 'key': None, 'skip_none': False, 'skip_check': True}
+        kwargs = {'subparser': parser, 'key': None, 'skip_none': False, 'skip_check': False}
         valid_flags = {'': None, 'comments': 'yaml_comments', 'skip_default': 'skip_default', 'skip_null': 'skip_none'}
         if value is not None:
             flags = value[0].split(',')
@@ -278,7 +292,8 @@ class _ActionPrintConfig(Action):
             subparser = parser.print_config.pop('subparser')
             if key is not None:
                 cfg = cfg[key]
-            sys.stdout.write(subparser.dump(cfg, **parser.print_config))
+            with lenient_check_context():
+                sys.stdout.write(subparser.dump(cfg, **parser.print_config))
             delattr(parser, 'print_config')
             parser.exit()
 
@@ -310,7 +325,7 @@ class _ActionConfigLoad(Action):
 
     def _load_config(self, value, parser):
         try:
-            cfg, cfg_path = _parse_value_or_config(value)
+            cfg, cfg_path = parse_value_or_config(value)
             if not isinstance(cfg, dict):
                 raise TypeError(f'Parser key "{self.dest}": Unable to load config "{value}"')
             with change_to_path_dir(cfg_path):
@@ -345,11 +360,11 @@ class _ActionHelpClassPath(Action):
 
     def update_init_kwargs(self, kwargs):
         if getattr(self._baseclass, '__origin__', None) == Union:
-            self._basename = '{'+', '.join(c.__name__ for c in self._baseclass.__args__ if c != NoneType)+'}'
+            self._basename = iter_to_set_str(c.__name__ for c in self._baseclass.__args__ if c != NoneType)
         else:
             self._basename = self._baseclass.__name__
         kwargs.update({
-            'metavar': 'CLASS',
+            'metavar': 'CLASS_PATH_OR_NAME',
             'default': SUPPRESS,
             'help': f'Show the help for the given subclass of {self._basename} and exit.',
         })
@@ -371,13 +386,13 @@ class _ActionHelpClassPath(Action):
             baseclasses = self._baseclass.__args__
         else:
             baseclasses = [baseclass]
-        if not any(_issubclass(val_class, b) for b in baseclasses):
+        if not any(is_subclass(val_class, b) for b in baseclasses):
             raise TypeError(f'{call_args[3]}: Class "{call_args[2]}" is not a subclass of {self._basename}')
         dest += '.init_args'
         subparser = import_object('jsonargparse.ArgumentParser')()
         subparser.add_class_arguments(val_class, dest, **self.sub_add_kwargs)
         _remove_actions(subparser, (_HelpAction, _ActionPrintConfig, _ActionConfigLoad))
-        args = self.get_extra_args(call_args[0].args)
+        args = self.get_args_after_opt(call_args[0].args)
         if args:
             subparser.parse_args(args)
             raise ParserError(f'Expected a nested --*.help option, got: {args}.')
@@ -385,7 +400,7 @@ class _ActionHelpClassPath(Action):
             subparser.print_help()
             call_args[0].exit()
 
-    def get_extra_args(self, args):
+    def get_args_after_opt(self, args):
         opt_str = self.option_strings[0]
         for num, arg in enumerate(args):
             parts = arg.split('=', 1)
@@ -461,7 +476,7 @@ class _ActionLink(Action):
         if target in parser.required_args:
             parser.required_args.remove(target)
         if is_target_subclass:
-            sub_add_kwargs = getattr(self.target[1], 'sub_add_kwargs')
+            sub_add_kwargs = getattr(self.target[1], 'sub_add_kwargs', {})
             if 'linked_targets' not in sub_add_kwargs:
                 sub_add_kwargs['linked_targets'] = set()
             subtarget = target.split('.init_args.', 1)[1]
@@ -610,7 +625,7 @@ class _ActionLink(Action):
         def del_taget_key(target_key):
             cfg.pop(target_key, None)
             parent_key, _ = split_key_leaf(target_key)
-            if '.' in target_key and is_empty_namespace(cfg.get(parent_key)):
+            if '.' in target_key and parent_key in cfg and not cfg[parent_key]:
                 del cfg[parent_key]
 
         for action in [a for a in parser._actions if isinstance(a, _ActionLink)]:
@@ -741,7 +756,7 @@ class ActionParser:
         def add_prefix(key):
             return re.sub('^--', '--'+prefix+'.', key)
 
-        required_args = set(prefix+'.'+x for x in subparser.required_args)
+        required_args = {prefix+'.'+x for x in subparser.required_args}
 
         option_string_actions = {}
         for key, action in filter_default_actions(subparser._option_string_actions).items():
@@ -1021,7 +1036,7 @@ class ActionPathList(Action, FilesCompleterMethod):
             value = []
             for path_list_file in path_list_files:
                 try:
-                    with sys.stdin if path_list_file == '-' else open(path_list_file, 'r') as f:
+                    with sys.stdin if path_list_file == '-' else open(path_list_file) as f:
                         path_list = [x.strip() for x in f.readlines()]
                 except FileNotFoundError as ex:
                     raise TypeError(f'Problems reading path list: {path_list_file} :: {ex}') from ex
