@@ -15,9 +15,9 @@ from .formatters import DefaultHelpFormatter, empty_help, formatter_context, get
 from .jsonnet import ActionJsonnet
 from .jsonschema import ActionJsonSchema
 from .loaders_dumpers import check_valid_dump_format, dump_using_format, get_loader_exceptions, loaders, load_value, load_value_context, yaml_load
-from .namespace import is_meta_key, Namespace, split_key, split_key_leaf, strip_meta
+from .namespace import dict_to_namespace, is_meta_key, Namespace, split_key, split_key_leaf, strip_meta
 from .signatures import is_pure_dataclass, SignatureArguments
-from .typehints import ActionTypeHint, is_class_object
+from .typehints import ActionTypeHint, is_subclass_spec
 from .typing import is_final_class
 from .actions import (
     ActionParser,
@@ -34,6 +34,7 @@ from .actions import (
     _is_action_value_list,
     filter_default_actions,
     parent_parsers,
+    previous_config,
 )
 from .optionals import (
     argcomplete_autocomplete,
@@ -50,8 +51,7 @@ from .util import (
     usage_and_exit_error_handler,
     change_to_path_dir,
     Path,
-    LoggerProperty,
-    _lenient_check_context,
+    lenient_check_context,
     lenient_check,
     return_parser_if_captured,
 )
@@ -60,7 +60,7 @@ from .util import (
 __all__ = ['ActionsContainer', 'ArgumentParser']
 
 
-class ActionsContainer(SignatureArguments, argparse._ActionsContainer, LoggerProperty):
+class ActionsContainer(SignatureArguments, argparse._ActionsContainer):
     """Extension of argparse._ActionsContainer to support additional functionalities."""
 
     _action_groups: Sequence['_ArgumentGroup']  # type: ignore
@@ -126,7 +126,7 @@ class ActionsContainer(SignatureArguments, argparse._ActionsContainer, LoggerPro
         are supported. Additionally it accepts:
 
         Args:
-            name: Name of the group. If set the group object will be included in the parser.groups dict.
+            name: Name of the group. If set, the group object will be included in the parser.groups dict.
 
         Returns:
             The group object.
@@ -165,13 +165,13 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
         env_prefix: Optional[str] = None,
         error_handler: Optional[Callable[['ArgumentParser', str], None]] = usage_and_exit_error_handler,
         formatter_class: Type[DefaultHelpFormatter] = DefaultHelpFormatter,
-        logger: Optional[Union[bool, Dict[str, str], logging.Logger]] = None,
+        logger: Union[bool, str, dict, logging.Logger] = False,
         version: Optional[str] = None,
         print_config: Optional[str] = '--print_config',
         parser_mode: str = 'yaml',
         dump_header: Optional[List[str]] = None,
         default_config_files: Optional[List[str]] = None,
-        default_env: bool = yaml_load(os.environ.get('JSONARGPARSE_DEFAULT_ENV', 'False')),
+        default_env: bool = False,
         default_meta: bool = True,
         **kwargs
     ):
@@ -183,10 +183,10 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
 
         Args:
             env_prefix: Prefix for environment variables.
-            error_handler: Handler for parsing errors, set to None to simply raise exception.
+            error_handler: Handler for parsing errors, if None raises :class:`.ParserError` exception.
             formatter_class: Class for printing help messages.
             logger: Configures the logger, see :class:`.LoggerProperty`.
-            version: Program version string to add --version argument.
+            version: Program version which will be printed by the --version argument.
             print_config: Add this as argument to print config, set None to disable.
             parser_mode: Mode for parsing configuration files: ``'yaml'``, ``'jsonnet'`` or ones added via :func:`.set_loader`.
             dump_header: Header to include as comment when dumping a config object.
@@ -234,7 +234,7 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
         namespace = argcomplete_namespace(caller, self, namespace)
 
         try:
-            with patch('argparse.Namespace', Namespace), _lenient_check_context(caller), ActionTypeHint.subclass_arg_context(self), load_value_context(self.parser_mode):
+            with patch('argparse.Namespace', Namespace), lenient_check_context(caller), ActionTypeHint.subclass_arg_context(self), load_value_context(self.parser_mode):
                 namespace, args = self._parse_known_args(args, namespace)
         except (argparse.ArgumentError, ParserError) as ex:
             self.error(str(ex), ex)
@@ -299,7 +299,7 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
             cfg = self.merge_config(cfg, self.get_defaults(skip_check=True))
             with load_value_context(self.parser_mode):
                 ActionTypeHint.apply_appends(self, cfg)
-            with _lenient_check_context():
+            with lenient_check_context():
                 ActionTypeHint.add_sub_defaults(self, cfg)
 
         if not (with_meta or (with_meta is None and self._default_meta)):
@@ -497,8 +497,7 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
         env: Optional[bool] = None,
         defaults: bool = True,
         with_meta: Optional[bool] = None,
-        _skip_check: bool = False,
-        _fail_no_subcommand: bool = True,
+        **kwargs,
     ) -> Namespace:
         """Parses a configuration file (yaml or jsonnet) given its path.
 
@@ -518,14 +517,15 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
         fpath = Path(cfg_path, mode=get_config_read_mode())
         with change_to_path_dir(fpath):
             cfg_str = fpath.get_content()
-            parsed_cfg = self.parse_string(cfg_str,
-                                           os.path.basename(cfg_path),
-                                           ext_vars,
-                                           env,
-                                           defaults,
-                                           with_meta=with_meta,
-                                           _skip_check=_skip_check,
-                                           _fail_no_subcommand=_fail_no_subcommand)
+            parsed_cfg = self.parse_string(
+                cfg_str,
+                os.path.basename(cfg_path),
+                ext_vars,
+                env,
+                defaults,
+                with_meta,
+                **kwargs,
+            )
 
         self._logger.info(f'Parsed {self.parser_mode} from path: {cfg_path}')
 
@@ -562,7 +562,7 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
         """
         try:
             with load_value_context(self.parser_mode):
-                cfg = self._load_config_parser_mode(cfg_str, cfg_path, ext_vars)
+                cfg = self._load_config_parser_mode(cfg_str, cfg_path, ext_vars, previous_config.get())
 
             parsed_cfg = self._parse_common(
                 cfg=cfg,
@@ -571,7 +571,7 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
                 with_meta=with_meta,
                 skip_check=_skip_check,
                 fail_no_subcommand=_fail_no_subcommand,
-                log_message=(f'Parsed {self.parser_mode} string.'),
+                log_message=f'Parsed {self.parser_mode} string.',
             )
 
         except (TypeError, KeyError) as ex:
@@ -585,6 +585,7 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
         cfg_str: str,
         cfg_path: str = '',
         ext_vars: Optional[dict] = None,
+        prev_cfg: Optional[Namespace] = None,
     ) -> Namespace:
         """Loads a configuration string (yaml or jsonnet) into a namespace.
 
@@ -600,10 +601,7 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
             cfg_dict = load_value(cfg_str, path=cfg_path, ext_vars=ext_vars)
         except get_loader_exceptions() as ex:
             raise TypeError(f'Problems parsing config :: {ex}') from ex
-
-        cfg = self._apply_actions(cfg_dict)
-
-        return cfg
+        return self._apply_actions(cfg_dict, prev_cfg=prev_cfg)
 
 
     def link_arguments(
@@ -652,7 +650,8 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
         """
         if 'description' not in kwargs:
             kwargs['description'] = 'For more details of each subcommand add it as argument followed by --help.'
-        subcommands: _ActionSubCommands = super().add_subparsers(dest=dest, **kwargs)  # type: ignore
+        with formatter_context(self):
+            subcommands: _ActionSubCommands = super().add_subparsers(dest=dest, **kwargs)  # type: ignore
         if required:
             self.required_args.add(dest)
         subcommands._required = required  # type: ignore
@@ -696,21 +695,23 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
         cfg = strip_meta(cfg)
         _ActionLink.strip_link_target_keys(self, cfg)
 
-        if not skip_check:
-            with load_value_context(self.parser_mode):
+        with load_value_context(self.parser_mode):
+            if not skip_check:
                 self.check_config(cfg)
 
-        with load_value_context(self.parser_mode):
             dump_kwargs = {'skip_check': skip_check, 'skip_none': skip_none}
             self._dump_cleanup_actions(cfg, self._actions, dump_kwargs)
 
-        cfg = cfg.as_dict()
+            cfg_dict = cfg.as_dict()
 
-        if skip_default:
-            self._dump_delete_default_entries(cfg, self.get_defaults().as_dict())
+            if skip_default:
+                defaults = self.get_defaults(skip_check=True)
+                _ActionLink.strip_link_target_keys(self, defaults)
+                self._dump_cleanup_actions(defaults, self._actions, {'skip_check': True, 'skip_none': skip_none})
+                self._dump_delete_default_entries(cfg_dict, defaults.as_dict())
 
         with formatter_context(self):
-            return dump_using_format(self, cfg, 'yaml_comments' if yaml_comments else format)
+            return dump_using_format(self, cfg_dict, 'yaml_comments' if yaml_comments else format)
 
 
     def _dump_cleanup_actions(self, cfg, actions, dump_kwargs, prefix=''):
@@ -738,12 +739,12 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
                 val = subcfg[key]
                 default = subdefaults[key]
                 class_object_val = None
-                if is_class_object(val):
+                if is_subclass_spec(val):
                     if val['class_path'] != default.get('class_path'):
                         parser = ActionTypeHint.get_class_parser(val['class_path'])
                         default = {'init_args': parser.get_defaults().as_dict()}
                     class_object_val = val
-                    val = val['init_args']
+                    val = val.get('init_args')
                     default = default.get('init_args')
                 if val == default:
                     del subcfg[key]
@@ -904,7 +905,7 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
             KeyError: If key or its default not defined in the parser.
         """
         action, _ = _find_parent_action_and_subcommand(self, dest)
-        if action is None or dest != action.dest or action.dest == argparse.SUPPRESS:
+        if action is None or dest != action.dest:
             raise KeyError(f'No action for destination key "{dest}" to get its default.')
 
         def check_suppressed_default():
@@ -1026,7 +1027,12 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
                 val = cfg[key]
                 action = _find_action(self, key)
                 if action is None:
-                    if _is_branch_key(self, key) or key.endswith('.class_path') or '.init_args' in key:
+                    if (
+                        _is_branch_key(self, key) or
+                        key.endswith('.class_path') or
+                        key.endswith('.dict_kwargs') or
+                        '.init_args' in key
+                    ):
                         continue
                     action = _find_parent_action(self, key, exclude=_ActionConfigLoad)
                     if action and not ActionTypeHint.is_subclass_typehint(action):
@@ -1076,7 +1082,7 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
                 components.append(action)
 
         if instantiate_groups:
-            skip = set(c.dest for c in components)
+            skip = {c.dest for c in components}
             groups = [g for g in self._action_groups if hasattr(g, 'instantiate_class') and g.dest not in skip]
             components.extend(groups)
 
@@ -1168,7 +1174,17 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
         return help_str
 
 
-    def _apply_actions(self, cfg: Union[Namespace, Dict[str, Any]], parent_key: str = '') -> Namespace:
+    def print_usage(self, file=None):
+        with formatter_context(self):
+            return super().print_usage(file)
+
+
+    def _apply_actions(
+        self,
+        cfg: Union[Namespace, Dict[str, Any]],
+        parent_key: str = '',
+        prev_cfg: Optional[Namespace] = None,
+    ) -> Namespace:
         """Runs _check_value_key on actions present in config."""
         if isinstance(cfg, dict):
             cfg = Namespace(cfg)
@@ -1179,6 +1195,12 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
             keys = [parent_key+'.'+k for k in cfg_branch.__dict__.keys()]
         else:
             keys = list(cfg.__dict__.keys())
+
+        if prev_cfg:
+            prev_cfg = prev_cfg.clone().update(dict_to_namespace(cfg))
+        else:
+            prev_cfg = cfg
+
         config_keys: Set[str] = set()
         num = 0
         while num < len(keys):
@@ -1197,8 +1219,8 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
                 continue
             action_dest = action.dest if subcommand is None else subcommand+'.'+action.dest
             value = cfg[action_dest]
-            with _lenient_check_context(), load_value_context(self.parser_mode):
-                value = self._check_value_key(action, value, action_dest, cfg)
+            with lenient_check_context(), load_value_context(self.parser_mode):
+                value = self._check_value_key(action, value, action_dest, prev_cfg)
             if isinstance(action, _ActionConfigLoad):
                 config_keys.add(action_dest)
                 keys.append(action_dest)
@@ -1206,8 +1228,7 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
         return cfg[parent_key] if parent_key else cfg
 
 
-    @staticmethod
-    def merge_config(cfg_from: Namespace, cfg_to: Namespace) -> Namespace:
+    def merge_config(self, cfg_from: Namespace, cfg_to: Namespace) -> Namespace:
         """Merges the first configuration into the second configuration.
 
         Args:
@@ -1217,14 +1238,8 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
         Returns:
             A new object with the merged configuration.
         """
-        del_keys = []
-        for key_class_path in [k for k in cfg_from.keys() if k.endswith('.class_path')]:
-            key_init_args = key_class_path[:-len('class_path')] + 'init_args'
-            if key_class_path in cfg_to and key_init_args in cfg_to and cfg_from[key_class_path] != cfg_to[key_class_path]:
-                del_keys.append(key_init_args)
         cfg = cfg_to.clone()
-        for key in reversed(del_keys):
-            del cfg[key]
+        ActionTypeHint.discard_init_args_on_class_path_change(self, cfg, cfg_from)
         cfg.update(cfg_from)
         return cfg
 
@@ -1331,6 +1346,9 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
     def default_env(self) -> bool:
         """Whether by default environment variables parsing is enabled.
 
+        If the JSONARGPARSE_DEFAULT_ENV environment variable is set to true or
+        false, that value will take precedence.
+
         :getter: Returns the current default environment variables parsing setting.
         :setter: Sets the default environment variables parsing setting.
 
@@ -1342,7 +1360,9 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
 
     @default_env.setter
     def default_env(self, default_env:bool):
-        if isinstance(default_env, bool):
+        if os.environ.get('JSONARGPARSE_DEFAULT_ENV', '').lower() in {'true', 'false'}:
+            self._default_env = yaml_load(os.environ.get('JSONARGPARSE_DEFAULT_ENV'))
+        elif isinstance(default_env, bool):
             self._default_env = default_env
         else:
             raise ValueError('default_env has to be a boolean.')
