@@ -53,7 +53,7 @@ class PercentTemplate(Template):
     (?P<braced>[_a-z][_a-z0-9]*)\)s|
     (?P<invalid>)
     )
-    '''
+    '''  # type: ignore
 
 
 class DefaultHelpFormatter(HelpFormatter):
@@ -86,9 +86,24 @@ class DefaultHelpFormatter(HelpFormatter):
            (action.default is not None or not is_required) and \
            (action.option_strings or action.nargs in {OPTIONAL, ZERO_OR_MORE}):
             help_str += (', ' if help_str else '') + 'default: %(default)s'
+        if isinstance(action, ActionTypeHint):
+            help_str += action.extra_help()
         return action_help + (' ('+help_str+')' if help_str else '')
 
 
+    def _format_usage(self, *args, **kwargs):
+        usage = super()._format_usage(*args, **kwargs)
+        parser = formatter_parser.get()
+        for key in parser.required_args:
+            try:
+                default = parser.get_default(key)
+            except KeyError:
+                default = None
+            if default is None and f'[--{key} ' in usage:
+                usage = re.sub(f'\\[(--{key} [^\\]]+)]', r'\1', usage, count=1)
+        return usage
+
+
     def _format_action_invocation(self, action):
         parser = formatter_parser.get()
         if action.option_strings == [] or action.default == SUPPRESS or not parser.default_env:
@@ -111,17 +126,20 @@ class DefaultHelpFormatter(HelpFormatter):
         type_str = self._get_type_str(action)
         if type_str is not None:
             params['type'] = type_str
+        orig_default = action.default
         if params.get('default') == SUPPRESS:
             del params['default']
         elif 'default' in params:
             defaults = formatter_defaults.get()
             if defaults is not None:
-                params['default'] = defaults.get(action.dest)
+                params['default'] = action.default = defaults.get(action.dest)
             if params['default'] is None:
                 params['default'] = 'null'
             elif isinstance(params['default'], Namespace):
                 params['default'] = params['default'].as_dict()
-        return PercentTemplate(self._get_help_string(action)).safe_substitute(params)
+        help_str = PercentTemplate(self._get_help_string(action)).safe_substitute(params)
+        action.default = orig_default
+        return help_str
 
 
     def _get_type_str(self, action):
@@ -165,7 +183,7 @@ class DefaultHelpFormatter(HelpFormatter):
             for group in parser._action_groups:
                 actions = filter_default_actions(group._group_actions)
                 actions = [a for a in actions if not isinstance(a, (_ActionConfigLoad, ActionConfigFile, _ActionSubCommands))]
-                keys = set(re.sub(r'\.?[^.]+$', '', a.dest) for a in actions)
+                keys = {re.sub(r'\.?[^.]+$', '', a.dest) for a in actions}
                 for key in keys:
                     full_key = prefix + key if key != '' else parser_key
                     group_titles[full_key] = group.title
@@ -247,7 +265,9 @@ class DefaultHelpFormatter(HelpFormatter):
 def get_env_var(parser_or_formatter: Union['ArgumentParser', DefaultHelpFormatter], action: Action) -> str:
     """Returns the environment variable for a given parser or formatter and action."""
     if isinstance(parser_or_formatter, DefaultHelpFormatter):
-        parser_or_formatter = formatter_parser.get()
-    env_var = (parser_or_formatter._env_prefix+'_' if parser_or_formatter._env_prefix else '') + action.dest
+        parser = formatter_parser.get()
+    else:
+        parser = parser_or_formatter
+    env_var = (parser.env_prefix+'_' if parser.env_prefix else '') + action.dest
     env_var = env_var.replace('.', '__').upper()
     return env_var
