@@ -10,30 +10,37 @@ import warnings
 from collections import defaultdict
 from contextlib import contextmanager
 from contextvars import ContextVar
-from typing import Any, Callable, Optional, Tuple, Union
+from functools import wraps
+from types import BuiltinFunctionType, FunctionType, ModuleType
+from typing import Any, Callable, Optional, Tuple, Type, TypeVar, Union
 
 from .loaders_dumpers import load_value
 from .optionals import (
-    url_support,
+    fsspec_support,
+    get_config_read_mode,
+    import_fsspec,
+    import_reconplogger,
     import_requests,
     import_url_validator,
-    fsspec_support,
-    import_fsspec,
-    get_config_read_mode,
+    reconplogger_support,
+    url_support,
 )
 from .type_checking import ArgumentParser
 
 
 __all__ = [
     'capture_parser',
+    'class_from_function',
     'LoggerProperty',
     'null_logger',
     'ParserError',
     'Path',
+    'register_unresolvable_import_paths',
     'usage_and_exit_error_handler',
 ]
 
 
+logging_levels = {'CRITICAL', 'ERROR', 'WARNING', 'INFO', 'DEBUG'}
 null_logger = logging.getLogger('jsonargparse_null_logger')
 null_logger.addHandler(logging.NullHandler())
 null_logger.parent = None
@@ -115,7 +122,7 @@ def identity(value):
     return value
 
 
-def _parse_value_or_config(value: Any, enable_path: bool = True) -> Tuple[Any, Optional['Path']]:
+def parse_value_or_config(value: Any, enable_path: bool = True) -> Tuple[Any, Optional['Path']]:
     """Parses yaml/json config in a string or a path"""
     cfg_path = None
     if enable_path and type(value) is str:
@@ -154,7 +161,7 @@ def usage_and_exit_error_handler(parser: 'ArgumentParser', message: str) -> None
         parser.exit(2)
 
 
-def _issubclass(cls, class_or_tuple):
+def is_subclass(cls, class_or_tuple):
     """Extension of issubclass that supports non-class argument."""
     return inspect.isclass(cls) and issubclass(cls, class_or_tuple)
 
@@ -187,11 +194,34 @@ def import_module_leaf(name: str):
     return module
 
 
+unresolvable_import_paths = {}
+
+
+def register_unresolvable_import_paths(*modules: ModuleType):
+    """Saves import paths of module objects for which its import path is unresolvable from the object alone.
+
+    Objects with unresolvable import paths have the __module__ attribute set to None.
+    """
+    for module in modules:
+        for val in vars(module).values():
+            if getattr(val, '__module__', None) is None and \
+               getattr(val, '__name__', None) and \
+               type(val) in {BuiltinFunctionType, FunctionType, Type}:
+                unresolvable_import_paths[val] = f'{module.__name__}.{val.__name__}'
+
+
 def get_import_path(value):
     """Returns the shortest dot import path for the given object."""
-    path = value.__module__ + '.' + value.__qualname__
-    if '.' in value.__module__:
-        module_parts = value.__module__.split('.')
+    module_path = getattr(value, '__module__', None)
+    if module_path is None:
+        path = unresolvable_import_paths.get(value)
+        if not path:
+            raise ValueError(f'Not possible to determine the import path for object {value}.')
+        module_path = path.rsplit('.', 1)[0]
+    else:
+        path = module_path + '.' + value.__qualname__
+    if '.' in module_path:
+        module_parts = module_path.split('.')
         for num in range(len(module_parts)):
             module_path = '.'.join(module_parts[:num+1])
             module = import_module_leaf(module_path)
@@ -222,8 +252,8 @@ lenient_check: ContextVar = ContextVar('lenient_check', default=False)
 
 
 @contextmanager
-def _lenient_check_context(caller=None):
-    t = lenient_check.set(False if caller == 'argcomplete' else True)
+def lenient_check_context(caller=None, lenient=True):
+    t = lenient_check.set(False if caller == 'argcomplete' else lenient)
     try:
         yield
     finally:
@@ -244,6 +274,13 @@ def change_to_path_dir(path: Optional['Path']):
             os.chdir(cwd)
 
 
+def iter_to_set_str(val, sep=','):
+    val = list(val)
+    if len(val) == 1:
+        return str(val[0])
+    return '{'+sep.join(str(x) for x in val)+'}'
+
+
 def indent_text(text: str) -> str:
     return text.replace('\n', '\n  ')
 
@@ -289,6 +326,40 @@ class DirectedGraph:
         order.insert(0, source)
 
 
+class ClassFromFunctionBase:
+    wrapped_function: Callable
+
+
+ClassType = TypeVar('ClassType')
+
+
+def class_from_function(func: Callable[..., ClassType]) -> Type[ClassType]:
+    """Creates a dynamic class which if instantiated is equivalent to calling func.
+
+    Args:
+        func: A function that returns an instance of a class. It must have a return type annotation.
+    """
+    func_return = inspect.signature(func).return_annotation
+    if isinstance(func_return, str):
+        caller_frame = inspect.currentframe().f_back  # type: ignore
+        func_return = caller_frame.f_locals.get(func_return) or caller_frame.f_globals.get(func_return)  # type: ignore
+        if func_return is None:
+            raise ValueError(f'Unable to dereference {func_return} the return type of {func}.')
+
+    @wraps(func)
+    def __new__(cls, *args, **kwargs):
+        return func(*args, **kwargs)
+
+    class ClassFromFunction(func_return, ClassFromFunctionBase):  # type: ignore
+        pass
+
+    ClassFromFunction.wrapped_function = func
+    ClassFromFunction.__new__ = __new__  # type: ignore
+    ClassFromFunction.__doc__ = func.__doc__
+    ClassFromFunction.__name__ = func.__name__
+    return ClassFromFunction
+
+
 class Path:
     """Stores a (possibly relative) path and the corresponding absolute path.
 
@@ -455,13 +526,51 @@ class Path:
             raise ValueError('Both modes "d" and "s" not possible.')
 
 
+def setup_default_logger(data, level, caller):
+    name = caller
+    if isinstance(data, str):
+        name = data
+    elif isinstance(data, dict) and 'name' in data:
+        name = data['name']
+    logger = logging.getLogger(name)
+    logger.parent = None
+    if len(logger.handlers) == 0:
+        handler = logging.StreamHandler()
+        handler.setFormatter(logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s'))
+        logger.addHandler(handler)
+    level = getattr(logging, level)
+    for handler in logger.handlers:
+        handler.setLevel(level)
+    return logger
+
+
+def parse_logger(logger: Union[bool, str, dict, logging.Logger], caller):
+    if not isinstance(logger, (bool, str, dict, logging.Logger)):
+        raise ValueError(f'Expected logger to be an instance of (bool, str, dict, logging.Logger), but got {logger}.')
+    if isinstance(logger, dict) and len(set(logger.keys())-{'name', 'level'}) > 0:
+        value = {k: v for k, v in logger.items() if k not in {'name', 'level'}}
+        raise ValueError(f'Unexpected data to configure logger: {value}.')
+    if logger is False:
+        return null_logger
+    level = 'WARNING'
+    if isinstance(logger, dict) and 'level' in logger:
+        level = logger['level']
+    if level not in logging_levels:
+        raise ValueError(f'Got logger level {level!r} but must be one of {logging_levels}.')
+    if (logger is True or (isinstance(logger, dict) and 'name' not in logger)) and reconplogger_support:
+        logger = import_reconplogger('parse_logger').logger_setup(level=level)
+    if not isinstance(logger, logging.Logger):
+        logger = setup_default_logger(logger, level, caller)
+    return logger
+
+
 class LoggerProperty:
     """Class designed to be inherited by other classes to add a logger property."""
 
-    def __init__(self):
+    def __init__(self, *args, logger: Union[bool, str, dict, logging.Logger] = False, **kwargs):
         """Initializer for LoggerProperty class."""
-        if not hasattr(self, '_logger'):
-            self.logger = None
+        self.logger = logger
+        super().__init__(*args, **kwargs)
 
 
     @property
@@ -471,48 +580,21 @@ class LoggerProperty:
         :getter: Returns the current logger.
         :setter: Sets the given logging.Logger as logger or sets the default logger
                  if given True/str(logger name)/dict(name, level), or disables logging
-                 if given False/None.
+                 if given False.
 
         Raises:
             ValueError: If an invalid logger value is given.
         """
         return self._logger
 
-
     @logger.setter
-    def logger(self, logger):
-        if logger is None or (isinstance(logger, bool) and not logger):
-            self._logger = null_logger
-        elif isinstance(logger, (bool, str, dict)) and logger:
-            levels = {'CRITICAL', 'ERROR', 'WARNING', 'INFO', 'DEBUG'}
-            level = logging.WARNING
-            if isinstance(logger, dict) and len(set(logger.keys())-{'name', 'level'}) > 0:
-                value = {k: v for k, v in logger.items() if k not in {'name', 'level'}}
-                raise ValueError(f'Unexpected logger value {value}.')
-            if isinstance(logger, dict) and 'level' in logger:
-                if logger['level'] not in levels:
-                    raise ValueError(f'Logger level must be one of {levels}.')
-                level = getattr(logging, logger['level'])
-            if isinstance(logger, bool) or (isinstance(logger, dict) and 'name' not in logger):
-                try:
-                    import reconplogger
-                    logger = reconplogger.logger_setup(level=level)
-                except (ImportError, ValueError):
-                    pass
-            if not isinstance(logger, logging.Logger):
-                name = type(self).__name__
-                if isinstance(logger, str):
-                    name = logger
-                elif isinstance(logger, dict) and 'name' in logger:
-                    name = logger['name']
-                logger = logging.getLogger(name)
-                if len(logger.handlers) == 0:
-                    handler = logging.StreamHandler()
-                    handler.setFormatter(logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s'))
-                    logger.addHandler(handler)
-                logger.setLevel(level)
-            self._logger = logger
-        elif not isinstance(logger, logging.Logger):
-            raise ValueError('Expected logger to be an instance of logging.Logger or bool or str or dict or None.')
-        else:
-            self._logger = logger
+    def logger(self, logger: Union[bool, str, dict, logging.Logger]):
+        if logger is None:
+            from .deprecated import deprecation_warning, logger_property_none_message
+            deprecation_warning(LoggerProperty, logger_property_none_message)
+            logger = False
+        self._logger = parse_logger(logger, type(self).__name__)
+
+
+if 'JSONARGPARSE_DEBUG' in os.environ:
+    os.environ['LOGGER_LEVEL'] = 'DEBUG'  # pragma: no cover
